package de.jottyfan.minecraft.quickiefabric.container;

import com.mojang.blaze3d.systems.RenderSystem;

import de.jottyfan.minecraft.quickiefabric.init.RegistryManager;
import de.jottyfan.minecraft.quickiefabric.items.ItemBackpack;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.client.gui.screen.ingame.HandledScreen;
import net.minecraft.client.gui.screen.ingame.ScreenHandlerProvider;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.text.Text;
import net.minecraft.text.TranslatableText;
import net.minecraft.util.Identifier;

/**
 * 
 * @author jotty
 *
 */
@Environment(EnvType.CLIENT)
public class BackpackScreen extends HandledScreen<BackpackScreenHandler>
		implements ScreenHandlerProvider<BackpackScreenHandler> {
	private final static Identifier TEXTURE = new Identifier(RegistryManager.QUICKIEFABRIC, "textures/gui/backpack.png");
	private final static Identifier SLOT_TEXTURE = new Identifier(RegistryManager.QUICKIEFABRIC, "textures/gui/slot.png");
	private final Integer containerHeight = 222;
	private final Integer containerWidth = 176;

	public BackpackScreen(BackpackScreenHandler handler, PlayerInventory inventory, Text text) {
		super(handler, inventory, new TranslatableText("container.quickiefabric.backpack"));
	}

	@Override
	protected void init() {
		super.init();
		this.titleX = (this.width - this.containerWidth) / 2;
	}

	private void drawSlots(MatrixStack matrices, int guiX, int guiY) {
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, SLOT_TEXTURE);
		for (int y = 0; y < (ItemBackpack.SLOTSIZE / 9); y++) {
			for (int x = 0; x < 9; x++) {
				this.drawTexture(matrices, guiX + 7 + (x * 18), guiY + 17 + (y * 18), 0, 0, 18, 18);
			}
		}
	}

	@Override
	public void render(MatrixStack matrices, int mouseX, int mouseY, float partialTicks) {
		this.renderBackground(matrices);
		super.render(matrices, mouseX, mouseY, partialTicks);
		this.drawMouseoverTooltip(matrices, mouseX, mouseY);
	}

	@Override
	protected void drawBackground(MatrixStack matrices, float delta, int mouseX, int mouseY) {
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, TEXTURE);
		int guiX = this.titleX;
		int guiY = (this.height - this.containerHeight) / 2;
		this.drawTexture(matrices, guiX, guiY, 0, 0, this.containerWidth, this.containerHeight);

		drawSlots(matrices, guiX, guiY);
	}

	@Override
	protected void drawForeground(MatrixStack matrixStack, int i, int j) {
		this.textRenderer.draw(matrixStack, this.title, 8.0F, -20.0F, 0xffcccccc); // αrgb
	}
}
