package de.jottyfan.minecraft.quickiefabric.init;

import java.util.Arrays;
import java.util.function.Predicate;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockDownEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockEastEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockNorthEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockSouthEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.DrillBlockWestEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.EmptyLavaHoarderBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.ItemHoarderBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.MonsterHoarderBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blockentity.QuickieFabricBlockEntity;
import de.jottyfan.minecraft.quickiefabric.blocks.QuickieBlocks;
import de.jottyfan.minecraft.quickiefabric.container.BackpackScreenHandler;
import de.jottyfan.minecraft.quickiefabric.event.BreakBlockCallback;
import de.jottyfan.minecraft.quickiefabric.event.EventBlockBreak;
import de.jottyfan.minecraft.quickiefabric.items.QuickieItems;
import de.jottyfan.minecraft.quickiefabric.tools.QuickieTools;
import net.fabricmc.fabric.api.biome.v1.BiomeModifications;
import net.fabricmc.fabric.api.biome.v1.BiomeSelectionContext;
import net.fabricmc.fabric.api.biome.v1.BiomeSelectors;
import net.fabricmc.fabric.api.client.itemgroup.FabricItemGroupBuilder;
import net.fabricmc.fabric.api.loot.v1.event.LootTableLoadingCallback;
import net.fabricmc.fabric.api.object.builder.v1.block.entity.FabricBlockEntityTypeBuilder;
import net.fabricmc.fabric.api.registry.FuelRegistry;
import net.fabricmc.fabric.api.screenhandler.v1.ScreenHandlerRegistry;
import net.minecraft.block.Block;
import net.minecraft.block.Blocks;
import net.minecraft.block.ComposterBlock;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.item.BlockItem;
import net.minecraft.item.Item;
import net.minecraft.item.ItemGroup;
import net.minecraft.item.ItemStack;
import net.minecraft.screen.ScreenHandlerType;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Identifier;
import net.minecraft.util.registry.BuiltinRegistries;
import net.minecraft.util.registry.Registry;
import net.minecraft.util.registry.RegistryEntry;
import net.minecraft.util.registry.RegistryKey;
import net.minecraft.world.gen.GenerationStep;
import net.minecraft.world.gen.YOffset;
import net.minecraft.world.gen.feature.ConfiguredFeature;
import net.minecraft.world.gen.feature.EmeraldOreFeatureConfig;
import net.minecraft.world.gen.feature.Feature;
import net.minecraft.world.gen.feature.OreConfiguredFeatures;
import net.minecraft.world.gen.feature.OreFeatureConfig;
import net.minecraft.world.gen.feature.PlacedFeature;
import net.minecraft.world.gen.placementmodifier.CountPlacementModifier;
import net.minecraft.world.gen.placementmodifier.HeightRangePlacementModifier;
import net.minecraft.world.gen.placementmodifier.SquarePlacementModifier;

/**
 *
 * @author jotty
 *
 */
public class RegistryManager {
	private static final Logger LOGGER = LogManager.getLogger(RegistryManager.class);

	public static final String QUICKIEFABRIC = "quickiefabric";

	public static final Identifier BACKPACK_IDENTIFIER = new Identifier(QUICKIEFABRIC, "backpack");
	public static final ScreenHandlerType<BackpackScreenHandler> BACKPACK_SCREEN_HANDLER = ScreenHandlerRegistry
			.registerExtended(RegistryManager.BACKPACK_IDENTIFIER, BackpackScreenHandler::new);

	@SuppressWarnings("unchecked")
	public static final ConfiguredFeature<?, ?> CF_ORESULPHOR = new ConfiguredFeature(Feature.ORE, new OreFeatureConfig(
			OreConfiguredFeatures.STONE_ORE_REPLACEABLES, QuickieBlocks.ORE_SULPHOR.getDefaultState(), 20));
	@SuppressWarnings("unchecked")
	public static final ConfiguredFeature<?, ?> CF_ORENETHERSULPHOR = new ConfiguredFeature(Feature.ORE,
			new OreFeatureConfig(OreConfiguredFeatures.NETHERRACK, QuickieBlocks.ORE_NETHER_SULPHOR.getDefaultState(), 24));
	@SuppressWarnings("unchecked")
	public static final ConfiguredFeature<?, ?> CF_ORESALPETER = new ConfiguredFeature(Feature.ORE, new OreFeatureConfig(
			OreConfiguredFeatures.STONE_ORE_REPLACEABLES, QuickieBlocks.ORE_SALPETER.getDefaultState(), 12));
	@SuppressWarnings("unchecked")
	public static final ConfiguredFeature<?, ?> CF_DIRTSALPETER = new ConfiguredFeature(Feature.REPLACE_SINGLE_BLOCK,
			new EmeraldOreFeatureConfig(Blocks.DIRT.getDefaultState(), QuickieBlocks.DIRT_SALPETER.getDefaultState()));
	@SuppressWarnings("unchecked")
	public static final ConfiguredFeature<?, ?> CF_SANDSALPETER = new ConfiguredFeature(Feature.REPLACE_SINGLE_BLOCK,
			new EmeraldOreFeatureConfig(Blocks.SAND.getDefaultState(), QuickieBlocks.SAND_SALPETER.getDefaultState()));
	@SuppressWarnings("unchecked")
	public static final ConfiguredFeature<?, ?> CF_ORESANDSALPETER = new ConfiguredFeature(Feature.REPLACE_SINGLE_BLOCK,
			new EmeraldOreFeatureConfig(Blocks.SANDSTONE.getDefaultState(),
					QuickieBlocks.ORE_SAND_SALPETER.getDefaultState()));
	
	public static final PlacedFeature PF_ORESULPHOR = new PlacedFeature(RegistryEntry.of(CF_ORESULPHOR),
			Arrays.asList(CountPlacementModifier.of(7), SquarePlacementModifier.of(),
					HeightRangePlacementModifier.uniform(YOffset.aboveBottom(0), YOffset.belowTop(0))));
	public static final PlacedFeature PF_ORENETHERSULPHOR = new PlacedFeature(RegistryEntry.of(CF_ORENETHERSULPHOR),
			Arrays.asList(CountPlacementModifier.of(10), SquarePlacementModifier.of(),
					HeightRangePlacementModifier.uniform(YOffset.aboveBottom(0), YOffset.belowTop(0))));
	public static final PlacedFeature PF_ORESALPETER = new PlacedFeature(RegistryEntry.of(CF_ORESALPETER),
			Arrays.asList(CountPlacementModifier.of(10), SquarePlacementModifier.of(),
					HeightRangePlacementModifier.uniform(YOffset.aboveBottom(0), YOffset.belowTop(0))));
	public static final PlacedFeature PF_DIRTSALPETER = new PlacedFeature(RegistryEntry.of(CF_DIRTSALPETER),
			Arrays.asList(CountPlacementModifier.of(4), SquarePlacementModifier.of(),
					HeightRangePlacementModifier.uniform(YOffset.aboveBottom(0), YOffset.belowTop(0))));
	public static final PlacedFeature PF_SANDSALPETER = new PlacedFeature(RegistryEntry.of(CF_SANDSALPETER),
			Arrays.asList(CountPlacementModifier.of(4), SquarePlacementModifier.of(),
					HeightRangePlacementModifier.uniform(YOffset.aboveBottom(0), YOffset.belowTop(0))));
	public static final PlacedFeature PF_ORESANDSALPETER = new PlacedFeature(RegistryEntry.of(CF_ORESANDSALPETER),
			Arrays.asList(CountPlacementModifier.of(4), SquarePlacementModifier.of(),
					HeightRangePlacementModifier.uniform(YOffset.aboveBottom(0), YOffset.belowTop(0))));

	public static final ItemGroup QUICKIEFABRIC_GROUP = FabricItemGroupBuilder
			.create(new Identifier(QUICKIEFABRIC, "all")).icon(() -> new ItemStack(QuickieItems.SPEEDPOWDER))
			.appendItems(stacks -> {
				stacks.add(new ItemStack(QuickieItems.SALPETER));
				stacks.add(new ItemStack(QuickieItems.SULPHOR));
				stacks.add(new ItemStack(QuickieItems.SPEEDPOWDER));
				stacks.add(new ItemStack(QuickieItems.LEVELUP));
				stacks.add(new ItemStack(QuickieItems.PENCIL));
				stacks.add(new ItemStack(QuickieItems.ROTTEN_FLESH_STRIPES));
				stacks.add(new ItemStack(QuickieItems.CARROTSTACK));
				stacks.add(new ItemStack(QuickieItems.COTTON));
				stacks.add(new ItemStack(QuickieItems.COTTONSEED));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_BROWN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_WHITE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_BLACK));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_BLUE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_CYAN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_GREEN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_PINK));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_RED));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_YELLOW));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_DARKGRAY));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_LIGHTGRAY));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_LIGHTGREEN));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_MAGENTA));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_ORANGE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_PURPLE));
				stacks.add(new ItemStack(QuickieItems.BACKPACK_LIGHTBLUE));
				stacks.add(new ItemStack(QuickieItems.BAG));
				stacks.add(new ItemStack(QuickieItems.STUB));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERAXE));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERPICKAXE));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERSHOVEL));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERHOE));
				stacks.add(new ItemStack(QuickieTools.SPEEDPOWDERWATERHOE));
				stacks.add(new ItemStack(QuickieBlocks.DIRT_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.ORE_NETHER_SULPHOR));
				stacks.add(new ItemStack(QuickieBlocks.ORE_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.ORE_SAND_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.ORE_SULPHOR));
				stacks.add(new ItemStack(QuickieBlocks.SAND_SALPETER));
				stacks.add(new ItemStack(QuickieBlocks.LAVAHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.EMPTYLAVAHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.ITEMHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.MONSTERHOARDER));
				stacks.add(new ItemStack(QuickieBlocks.KELPSTACK));
				stacks.add(new ItemStack(QuickieBlocks.BLOCKSULPHOR));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_DOWN));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_EAST));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_SOUTH));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_WEST));
				stacks.add(new ItemStack(QuickieBlocks.DRILL_NORTH));
				stacks.add(new ItemStack(QuickieBlocks.DRILLSTOP));
			}).build();

	private static final void registerBlock(Block block, String name) {
		Registry.register(Registry.BLOCK, new Identifier(QUICKIEFABRIC, name), block);
		Registry.register(Registry.ITEM, new Identifier(QUICKIEFABRIC, name),
				new BlockItem(block, new Item.Settings().group(RegistryManager.QUICKIEFABRIC_GROUP)));
	}

	private static final void registerItem(Item item, String name) {
		Registry.register(Registry.ITEM, new Identifier(QUICKIEFABRIC, name), item);
	}

	public static final <T extends BlockEntity> BlockEntityType<? extends T> registerBlockEntity(String name,
			FabricBlockEntityTypeBuilder.Factory<? extends T> supplier, Block... blocks) {
		String fullname = new StringBuilder().append(QUICKIEFABRIC).append(":").append(name).toString();
		FabricBlockEntityTypeBuilder<? extends T> builder = FabricBlockEntityTypeBuilder.create(supplier, blocks);
		BlockEntityType<? extends T> blockEntityType = builder.build(null);
		return Registry.register(Registry.BLOCK_ENTITY_TYPE, fullname, blockEntityType);
	}

	@SuppressWarnings("unchecked")
	public static final void registerBlockEntities() {
		LOGGER.debug("registering quickiefabric block entities");
		QuickieFabricBlockEntity.ITEMHOARDER = (BlockEntityType<ItemHoarderBlockEntity>) registerBlockEntity(
				"itemhoarderblockentity", ItemHoarderBlockEntity::new, QuickieBlocks.ITEMHOARDER);
		QuickieFabricBlockEntity.MONSTERHOARDER = (BlockEntityType<MonsterHoarderBlockEntity>) registerBlockEntity(
				"monsterhoarderblockentity", MonsterHoarderBlockEntity::new, QuickieBlocks.MONSTERHOARDER);
		QuickieFabricBlockEntity.EMPTYLAVAHOARDER = (BlockEntityType<EmptyLavaHoarderBlockEntity>) registerBlockEntity(
				"emptylavahoarderblockentity", EmptyLavaHoarderBlockEntity::new, QuickieBlocks.EMPTYLAVAHOARDER,
				QuickieBlocks.LAVAHOARDER);
		QuickieFabricBlockEntity.DRILL_DOWN = (BlockEntityType<DrillBlockDownEntity>) registerBlockEntity("drillblockdownentity",
				DrillBlockDownEntity::new, QuickieBlocks.DRILL_DOWN);
		QuickieFabricBlockEntity.DRILL_EAST = (BlockEntityType<DrillBlockEastEntity>) registerBlockEntity("drillblockeastentity",
				DrillBlockEastEntity::new, QuickieBlocks.DRILL_EAST);
		QuickieFabricBlockEntity.DRILL_SOUTH = (BlockEntityType<DrillBlockSouthEntity>) registerBlockEntity("drillblocksouthentity",
				DrillBlockSouthEntity::new, QuickieBlocks.DRILL_SOUTH);
		QuickieFabricBlockEntity.DRILL_WEST = (BlockEntityType<DrillBlockWestEntity>) registerBlockEntity("drillblockwestentity",
				DrillBlockWestEntity::new, QuickieBlocks.DRILL_WEST);
		QuickieFabricBlockEntity.DRILL_NORTH = (BlockEntityType<DrillBlockNorthEntity>) registerBlockEntity("drillblocknorthentity",
				DrillBlockNorthEntity::new, QuickieBlocks.DRILL_NORTH);
	}

	public static final void registerBlocks() {
		LOGGER.debug("registering quickiefabric blocks");
		registerBlock(QuickieBlocks.DIRT_SALPETER, "dirtsalpeter");
		registerBlock(QuickieBlocks.ORE_NETHER_SULPHOR, "orenethersulphor");
		registerBlock(QuickieBlocks.ORE_SALPETER, "oresalpeter");
		registerBlock(QuickieBlocks.ORE_SAND_SALPETER, "oresandsalpeter");
		registerBlock(QuickieBlocks.ORE_SULPHOR, "oresulphor");
		registerBlock(QuickieBlocks.SAND_SALPETER, "sandsalpeter");
		registerBlock(QuickieBlocks.LAVAHOARDER, "lavahoarder");
		registerBlock(QuickieBlocks.EMPTYLAVAHOARDER, "emptylavahoarder");
		registerBlock(QuickieBlocks.ITEMHOARDER, "itemhoarder");
		registerBlock(QuickieBlocks.MONSTERHOARDER, "monsterhoarder");
		registerBlock(QuickieBlocks.KELPSTACK, "kelpstack");
		registerBlock(QuickieBlocks.COTTONPLANT, "cottonplant");
		registerBlock(QuickieBlocks.BLOCKSULPHOR, "blocksulphor");
		registerBlock(QuickieBlocks.DRILL_DOWN, "drill");
		registerBlock(QuickieBlocks.DRILL_EAST, "drilleast");
		registerBlock(QuickieBlocks.DRILL_SOUTH, "drillsouth");
		registerBlock(QuickieBlocks.DRILL_WEST, "drillwest");
		registerBlock(QuickieBlocks.DRILL_NORTH, "drillnorth");
		registerBlock(QuickieBlocks.DRILLSTOP, "drillstop");
	}

	public static final void registerItems() {
		LOGGER.debug("registering quickiefabric items");
		registerItem(QuickieItems.SPEEDPOWDER, "speedpowder");
		registerItem(QuickieItems.LEVELUP, "levelup");
		registerItem(QuickieItems.PENCIL, "pencil");
		registerItem(QuickieItems.SALPETER, "salpeter");
		registerItem(QuickieItems.SULPHOR, "sulphor");
		registerItem(QuickieItems.ROTTEN_FLESH_STRIPES, "rotten_flesh_stripes");
		registerItem(QuickieItems.CARROTSTACK, "carrotstack");
		registerItem(QuickieItems.COTTON, "cotton");
		registerItem(QuickieItems.COTTONSEED, "cottonseed");
		registerItem(QuickieItems.BACKPACK_BROWN, "backpack_brown");
		registerItem(QuickieItems.BACKPACK_WHITE, "backpack_white");
		registerItem(QuickieItems.BACKPACK_BLACK, "backpack_black");
		registerItem(QuickieItems.BACKPACK_BLUE, "backpack_blue");
		registerItem(QuickieItems.BACKPACK_CYAN, "backpack_cyan");
		registerItem(QuickieItems.BACKPACK_GREEN, "backpack_green");
		registerItem(QuickieItems.BACKPACK_PINK, "backpack_pink");
		registerItem(QuickieItems.BACKPACK_RED, "backpack_red");
		registerItem(QuickieItems.BACKPACK_YELLOW, "backpack_yellow");
		registerItem(QuickieItems.BACKPACK_DARKGRAY, "backpack_darkgray");
		registerItem(QuickieItems.BACKPACK_LIGHTGRAY, "backpack_lightgray");
		registerItem(QuickieItems.BACKPACK_LIGHTGREEN, "backpack_lightgreen");
		registerItem(QuickieItems.BACKPACK_MAGENTA, "backpack_magenta");
		registerItem(QuickieItems.BACKPACK_ORANGE, "backpack_orange");
		registerItem(QuickieItems.BACKPACK_PURPLE, "backpack_purple");
		registerItem(QuickieItems.BACKPACK_LIGHTBLUE, "backpack_lightblue");
		registerItem(QuickieItems.BAG, "bag");
		registerItem(QuickieItems.STUB, "stub");

		ComposterBlock.ITEM_TO_LEVEL_INCREASE_CHANCE.put(QuickieItems.COTTONSEED, 0.5f);
		ComposterBlock.ITEM_TO_LEVEL_INCREASE_CHANCE.put(QuickieItems.COTTON, 0.75f);

		FuelRegistry.INSTANCE.add(QuickieItems.SULPHOR, 200);
		FuelRegistry.INSTANCE.add(QuickieBlocks.BLOCKSULPHOR, 2000);
	}

	public static final void registerTools() {
		LOGGER.debug("registering quickiefabric tools");
		registerItem(QuickieTools.SPEEDPOWDERAXE, "speedpowderaxe");
		registerItem(QuickieTools.SPEEDPOWDERPICKAXE, "speedpowderpickaxe");
		registerItem(QuickieTools.SPEEDPOWDERSHOVEL, "speedpowdershovel");
		registerItem(QuickieTools.SPEEDPOWDERHOE, "speedpowderhoe");
		registerItem(QuickieTools.SPEEDPOWDERWATERHOE, "speedpowderwaterhoe");
	}

	public static final void registerContainer() {
	}

	public static final void registerEvents() {
		LOGGER.debug("registering quickiefabric events");
		BreakBlockCallback.EVENT.register((world, blockPos, blockState, playerEntity) -> {
			new EventBlockBreak().doBreakBlock(world, blockPos, blockState, playerEntity);
			return ActionResult.SUCCESS;
		});
	}

	/**
	 * register ore generation feature
	 *
	 * @param name    the name for the identifier
	 * @param feature the feature
	 */
	private static final void registerFeature(String name, ConfiguredFeature<?, ?> conf, PlacedFeature feature,
			Predicate<BiomeSelectionContext> predicate) {
		RegistryKey<ConfiguredFeature<?, ?>> identifier = RegistryKey.of(Registry.CONFIGURED_FEATURE_KEY,
				new Identifier(QUICKIEFABRIC, name));
		RegistryKey<PlacedFeature> identifierP = RegistryKey.of(Registry.PLACED_FEATURE_KEY,
				new Identifier(QUICKIEFABRIC, name));
		Registry.register(BuiltinRegistries.PLACED_FEATURE, identifier.getValue(), feature);
		Registry.register(BuiltinRegistries.CONFIGURED_FEATURE, identifier.getValue(), conf);
		BiomeModifications.addFeature(predicate, GenerationStep.Feature.UNDERGROUND_ORES, identifierP);
	}

	/**
	 * register ore generation features
	 */
	public static final void registerFeatures() {
		Predicate<BiomeSelectionContext> nether = BiomeSelectors.foundInTheNether();
		Predicate<BiomeSelectionContext> overworld = BiomeSelectors.foundInOverworld();

		registerFeature("feature_orenethersulphor", CF_ORENETHERSULPHOR, PF_ORENETHERSULPHOR, nether);
		registerFeature("feature_oresalpeter", CF_ORESALPETER, PF_ORESALPETER, overworld);
		registerFeature("feature_oresulphor", CF_ORESULPHOR, PF_ORESULPHOR, overworld);
		registerFeature("feature_dirtsalpeter", CF_DIRTSALPETER, PF_DIRTSALPETER, overworld);
		registerFeature("feature_sandsalpeter", CF_SANDSALPETER, PF_SANDSALPETER, overworld);
		registerFeature("feature_oresandsalpeter", CF_ORESANDSALPETER, PF_ORESANDSALPETER, overworld);
	}

	public static final void registerLootings() {
		LootTableLoadingCallback.EVENT.register((resourceManager, lootManager, id, supplier, setter) -> {
			if (isGrass(id)) {
				supplier.withPool(LootHelper.build(1, QuickieItems.COTTONSEED, 0.125f).build());
				supplier.withPool(LootHelper.build(2, QuickieItems.SALPETER, 0.012f).build());
			}
		});
	}

	private final static boolean isGrass(Identifier id) {
		return id.equals(new Identifier("blocks/grass")) || id.equals(new Identifier("blocks/fern"))
				|| id.equals(new Identifier("blocks/tall_grass"));
	}

}
